# -*- coding: utf-8 -*-
"""
Created on Fri Apr 17 13:12:17 2020

@author: James Edgar McCutcheon
"""

import numpy as np
import os

def remcheck(val, range1, range2):
    """ Checks whether value is within range of two decimals.

    Parameters
    ----------
    val : float
        Value to be checked.
    range1 : float
        Decimal 1.
    range2 : float
        Decimal 2.

    Returns
    -------
    bool
        Result of check.

    """
    
    if (range1 < range2):
        if (val > range1) and (val < range2):
            return True
        else:
            return False
    else:
        if (val > range1) or (val < range2):
            return True
        else:
            return False
    
def random_array(dims,n, multiplier = 10):
    """ Creates random array of values. Useful for checking plot functions, e.g. barscatter.

    Parameters
    ----------
    dims : int
        Number of dimensions.
    n : int
        Number of data points in each data set.
    multiplier : int or float, optional
        Value to multiply random data points by. The default is 10.

    Returns
    -------
    data : List of lists
        Random data array as a list of lists with appropriate dimensions.
    """
    
    data = []
    import numpy as np
    try:
        if len(dims) == 2:
            data = np.empty((dims), dtype=np.object)        
            for i in range(np.shape(data)[0]):
                for j in range(np.shape(data)[1]):
                    data[i][j] = np.random.random((n))*multiplier
        elif len(dims) > 2:
            print('Too many dimensions!')
            return
        
        elif len(dims) == 1:
            data = np.empty((dims), dtype=np.object)
            for i,j in enumerate(data):
                data[i] = np.random.random((n))*multiplier
    except TypeError:
        print('Dimensions need to be in a list or matrix')
        return

    return data

def getuserhome():
    """ Returns path to user's home directory"""
    
    path = os.environ['USERPROFILE']
    return path

def flatten_list(listoflists):
    """ Flattens list of lists into a 1D list"""
    
    try:
        flat_list = [item for sublist in listoflists for item in sublist]
        return flat_list
    except:
        print('Cannot flatten list. Maybe is in the wrong format. Returning empty list.')
        return []
    
def discrete2continuous(onset, offset=[], nSamples=[], fs=[]):
    """
    Takes timestamp data (e.g. licks) that can include offsets as well as onsets,
    and returns a digital on/off array (y) as well as the x output.

    Parameters
    ----------
    onset : List or 1D array
        Timestamps of onsets.
    offset : List or 1D array, optional
        Timestamps of offsets. The default is [].
    nSamples : Int, optional
        Number of samples. The default is [].
    fs : Float, optional
        Sample frequency. The default is [].

    Returns
    -------
    outputx : List
        x values for y-array.
    outputy : List
        On / off array.
        
    Notes
    -------
    The number of samples (nSamples) and sample frequency (fs) can be input or if they are not (default) it will attempt to calculate them based on the timestamp data.
    Not been fully stress-tested yet.

    """

    try:
        fs = int(fs)
    except TypeError:
        isis = np.diff(onset)
        fs = int(1 / (min(isis)/2)) 
    
    if len(nSamples) == 0:
        nSamples = int(fs*max(onset))    
    
    outputx = np.linspace(0, nSamples/fs, nSamples)
    outputy = np.zeros(len(outputx))
    
    if len(offset) == 0:
        for on in onset:
            idx = (np.abs(outputx - on)).argmin()
            outputy[idx] = 1
    else:
        for i, on in enumerate(onset):
            start = (np.abs(outputx - on)).argmin()
            stop = (np.abs(outputx - offset[i])).argmin()
            outputy[start:stop] = 1

    return outputx, outputy

def findpercentilevalue(data, percentile):
    """ Finds percentile value in a specified data set.
    
    Parameters
    ------------
    data : list or 1D array
    percentile : float
        Must be between 0 and 1.
    
    Returns
    ---------
    value : int
        Value in data set closest to percentile
    
    """

    if (0 < percentile) and (percentile <= 1):
        position = int(percentile*len(data))
    else:
        print('Value for percentile must be between 0 and 1')
        return

    sorteddata = np.sort(data)
    value = sorteddata[position-1]
    
    return value

def logical_subset(data, logical, condition=True):
    """ Returns logical subset."""
    
    if condition:
        return [d for d, L in zip(data, logical) if L]
    else:
        return [d for d, L in zip(data, logical) if not L]
    
def find_overlap(i1, i2, if_none=None):
    """
    Finds the amount of overlap between two intervals. Useful for looking at
    time series when you only have onset and offset times.

    Parameters
    ----------
    i1 : Tuple or list of floats or integers with length=2
        Interval 1.
    i2 : Tuple or list of floats or integers with length=2
        Interval 2.
    if_none : Any, optional
        Value that will be returned if intervals do not overlap. The default is None.

    Returns
    -------
    Float, if there is overlap between intervals. Otherwise, depends on value
    of if_none variable but default is None
        Amount of overlap.

    """
    if (len(i1) != 2) or (len(i2) !=2):
        print("Intervals cannot be compared as they do not include start and stop.")
        return
    
    if (i1[1] < i2[0]) or (i1[0] > i2[1]):
        return if_none
    else:
        return min(i1[1], i2[1]) - max(i1[0], i2[0])

def download_data(url):
    """
    Downloads zipped data from a remote URL and unzips. Unzipped data
    is in a new directory in the current working directory called 'data'.
    
    Parameters
    ------------
    url : String with url for data to be retrieved from
    
    """
    import os
    print(os.getcwd())
    if not os.path.exists('data.zip'):
        import urllib.request
        import sys
        import time

        print('downloading data...')

        def reporthook(count, block_size, total_size):
            global start_time
            if count == 0:
                start_time = time.time()
                return
            duration = time.time() - start_time
            progress_size = int(count * block_size)
            if duration > 0:
                speed = int(progress_size / (1024 * duration))
                percent = min(int(count * block_size * 100 / total_size), 100)
                sys.stdout.write("\r...%d%%, %d MB, %d KB/s, %d seconds elapsed" %
                                (percent, progress_size / (1024 * 1024), speed, duration))
                sys.stdout.flush()

        urllib.request.urlretrieve(url, '.\\data.zip', reporthook)

    try:
        print('unzipping data...')
        import zipfile
        zip_ref = zipfile.ZipFile('data.zip', 'r')
        zip_ref.extractall('.\\data')
        zip_ref.close()
        os.remove('data.zip')
    except:
        print('problem with zip, downloading again')
        os.remove('data.zip')
        return download_demo_data()
    
    print("Datafiles ready")


from trompy.barscatter import barscatter # ,  prep_data, data2obj1D, data2obj2D, xyspacer
from trompy.medfilereader import medfilereader, medfilereader_licks, isnumeric, checknsessions, tstamp_to_tdate
from trompy.metafile_utils import metafilemaker, metafilereader
from trompy.snipper_utils import processdata, snipper, mastersnipper, zscore, findnoise, removenoise, med_abs_dev, makerandomevents, time2samples, event2sample, resample_snips
from trompy.general_utils import remcheck, random_array, getuserhome, flatten_list, discrete2continuous, findpercentilevalue, logical_subset, find_overlap, download_data
from trompy.lick_utils import lickCalc, removeshortbursts, calculate_burst_prob, weib_davis, fit_weibull
from trompy.stats_utils import sidakcorr, mean_and_sem, bonferroni_corrected_ttest
from trompy.fig_utils import setsameaxislimits, invisible_axes, shadedError, ax2prop, lighten_color, get_violinstats
from trompy.lick_figs import licklengthFig, iliFig, burstlengthFig, ibiFig, burstprobFig, sessionlicksFig
from trompy.trials_figs import trialsFig, trialsMultFig, trialsShadedFig, trialsMultShadedFig, trialstiledFig, makeheatmap
from trompy.roc_utils import rocN, rocshuf, nanroc, run_roc_comparison, plot_ROC_and_line
from trompy.merge_fed_files import merge_files, parse_args


# -*- coding: utf-8 -*-
"""
Created on Tue Apr 21 10:24:11 2020

@author: James Edgar McCutcheon
"""

import numpy as np
import time
import matplotlib.gridspec as gridspec
from matplotlib.colors import LinearSegmentedColormap
from trompy import flatten_list, logical_subset, shadedError


def rocN(x,y,N=100):
    """ Function to calculate ROC based on MATLAB function
    
    Parameters
    -----------
    x, y : Arrays or lists of numerical data
        Data to be compared.
    N : Int, optional
        Number of bins to use for calculation. Deafault is 100.
        
    Returns
    --------
    a : Float
        ROC value. Number between 0 and 1.
    """
    if len(x) > 0 and len(y) >0:
        pass
    else:
        print("x and/or y are incorrect form")
        return np.NaN
    
    if len(x)<3 or len(y)<3:
        print('Too few trials for roc analysis!')
        return np.NaN
    
    zlo = np.min([np.min(x), np.min(y)])
    zhi = np.max([np.max(x), np.max(y)])
    z = np.linspace(zlo, zhi, N)

    fa, hit = [], []
    
    for i in range(N):
        fa.append(np.count_nonzero(y > z[i])) # faster than np.sum
        hit.append(np.count_nonzero(x > z[i]))
        
    fa.reverse()
    hit.reverse()
    
    fa = np.divide(fa, len(y))
    hit = np.divide(hit, len(x))
    
    fa[0], fa[-1] = 0, 1
    hit[0], hit[-1] = 0, 1
    
    a = np.trapz(fa, hit)
    
    return a
    
def rocshuf(x,y,nsims=10):
    """
    Performs ROC on shuffled data to obtain p-vlaue (probability) of observed ROC value
    
    Parameters
    -----------
    x, y : Arrays or lists of numerical data
        Data to be compared
    nsims : Int, optional
        Number of simulations to run. Precision of p-value will be increased by using a larger number (e.g. nsims=100 gives p-value to 0.01 precision). Default is 10.
        
    Returns
    ---------
    roc0 : Float
        ROC value. Number between 0 and 1.
    p : Float
        p-value.

    """
    z = x + y
    b = [True for val in x] + [False for val in y]
    n0 = len(b)

    roc0 = rocN(logical_subset(z, b), logical_subset(z, b, condition=False))
    
    a=[]
    for sim in range(nsims):
        I = np.random.permutation(n0)
        B = [b[idx] for idx in I]
        a.append(rocN(logical_subset(z, B), logical_subset(z, B, condition=False)))
 
    absa = np.abs(roc0 -0.5)
    p1 = len([val for val in a if val >= 0.5+absa])
    p2 = len([val for val in a if val <= 0.5-absa])
    
    p = p1/nsims + p2/nsims
    
    return roc0, p

def nanroc(x, y, N=100, min4roc=4, n4shuf=100):
    """
    Performs ROC analysis on multiple arrays or columns of data, e.g. for examining a time series.
    
    Parameters
    -----------
    x, y : Arrays or lists of numerical data
        Data to be compared
    N : Int, optional
        Number of bins for calculating ROC. Not currently used. Default is 100.
    min4roc : Int, optional
        Minimum number of data points for comparison. Not currently used. Default is 4.
    n4shuf : Int, optional
        Number of shuffled simulations to run to derive p-value. Precision of p-value will be increased by using a larger number (e.g. n4shuf=100 gives p-value to 0.01 precision). Default is 100.
        
    Returns
    ---------
    a : List of floats
        ROC values corresponding to each column in input data.
    p : List of floats
        p-values corresponding to each column in input data.
    """
 
    # checks dimensions of matrices
    if np.shape(x)[1] != np.shape(y)[1]:
        print('nanroc: matrices must have the same number of columns')
    
    a=[]
    p=[]
    
    for idx in range(np.shape(x)[1]):
        print(f"Analysing column {idx}")
        x0 = [val[idx] for val in x]
        x1 = [val[idx] for val in y]
        
        a0, p0 = rocshuf(x0, x1, nsims=n4shuf)
        
        a.append(a0)
        p.append(p0)
        
    return a, p

def run_roc_comparison(data, n4shuf=10, timer=True, savedata=""):
    """ Function to run ROC analysis with option for timing and saving resulting data
    
    Parameters
    -----------
    data : List or array of numerical values
        Two distributions to be compared. Normally should be of the shape (2,x,y) where x is number of trials and can be different
        between each array and y is bins and should be identical. Example, data[0] can be 300x20 list of lists or array and data[1] can
        be 360x20.
    n4shuf : Int, optional
        Number of times to repeat roc with shuffled values to calculate p-values. Default is 10, so that it is fast to run, but for accurate p-values should run 2000 times
    timer : Bool, optional
        Prints time taken if True
    savedata : Str, optional
        Filename for saving data. If empty, does not save anything. Default is ""
    
    Returns
    a : List of floats
        ROC values corresponding to each column in input data.
    p : List of floats
        p-values corresponding to each column in input data.
    
    """
    
    if timer: start_time = time.time()

    a, p = nanroc(data[0], data[1], n4shuf=n4shuf)
    
    if timer: print(f"--- Total ROC analysis took {(time.time() - start_time)} seconds ---")
    
    if len(savedata)>0:
        try:       
            pickle_out = open(savedata, 'wb')
            dill.dump([a, p, data], pickle_out)
            pickle_out.close()
        except:
            print("Cannot save. Check filename.")

    return a, p

def plot_ROC_and_line(f, a, p, snips1, snips2,
                      cdict=['grey', 'white', 'red'],
                      colors=['grey', 'red'],
                      labels=["", ""],
                      labeloffset_y=0,
                      labeloffset_x=0,
                      ylabel='',
                      xlabel='',
                      gridspec_dict=None,
                      ):
    """
    Makes ROC figure. Originally used to produce figures in Peters et al (2021)
    
    Parameters
    -----------
    f : Matplotlib figure object
        
    a, p : Lists of floats
        ROC values and p-values from ROC analysis
    snips1, snips2 : Lists or arrays of floats
        Binned data corresponding to the comparison that produced a and p
    cdict : List of 3 strings, optional
        Colors representing mapping of colors for representation of ROC values
    colors : List of 2 strings, optional
        Colors representing data from snips1 and snips2
    labels : List of 2 strings, optional
        Labels for snips1 and snips2
    labeloffset_y : Float, optional
        Specifies y label offset. Default is 0.
    labeloffset_x : Float, optional
        Specifies x label offset. Default is 0.
    ylabel : Str, optional
        Specifies y label
    xlabel : Str, optional
        Specifies x label
    gridspec_dict : Dict, optional
        Allows specification of figure and customisation.
        
    Returns
    ----------
    f, ax : Matplotlib figure and axis
    """
    
    ax=[]
    
    if type(gridspec_dict) == dict:
        try:
            gs_spec = gridspec_dict['gs_spec']
            gsx = gridspec_dict['gsx']
            gsy = gridspec_dict['gsy']
    
            gs = gs_spec[gsx, gsy].subgridspec(2,2, height_ratios=[0.25, 1], width_ratios=[1, 0.05], wspace=0.05)
        except:
            print('Problem with gridspec_dict when plotting ROC so ignoring it. Check it has correct keys.')
            return
    else:   
        gs=gridspec.GridSpec(2,2, figure=f, height_ratios=[0.25, 1], width_ratios=[1, 0.05], wspace=0.05,
                            bottom=0.2, right=0.75, left=0.15)
    
    ax.append(f.add_subplot(gs[0, 0]))
    
    # Creates colormap for ROC

    heatmap_color_scheme = LinearSegmentedColormap.from_list('rocmap', cdict)
    
    roc_for_plot = a + [0]
    xlen=len(snips1[0])
    xvals=np.arange(-0.5,xlen+0.5)
    yvals=[1, 2]
    xx, yy = np.meshgrid(xvals, yvals)
        
    mesh = ax[0].pcolormesh(xx, yy, [roc_for_plot, roc_for_plot], cmap=heatmap_color_scheme, shading = 'flat')
    mesh.set_clim([0, 1])
    
    threshold = 0.05/len(p)
    sigpoints = np.array([pval < threshold for pval in p], dtype=bool)
    
    if sum(sigpoints) > 0:
        xdata = [x for x, L in zip(range(len(sigpoints)), sigpoints) if L]
        ydata = logical_subset(a, sigpoints)
        ax[0].scatter(xdata, [2.5]*len(xdata), color='k', marker='.', clip_on=False)
    else:
        ax[0].scatter(2, 2.5, color='white', marker='.', clip_on=False)
    
    ax[0].text(-1, 2.5, 'p<0.05', va='center', ha='right')
    
    ax[0].spines['top'].set_visible(False)
    ax[0].spines['right'].set_visible(False)
    ax[0].spines['bottom'].set_visible(False)
    ax[0].spines['left'].set_visible(False)
    ax[0].set_xticks([])
    ax[0].set_yticks([])
     
    cbar_ax = f.add_subplot(gs[0,1])   
    cbar = f.colorbar(mesh, cax=cbar_ax, ticks=[0, 1], label='ROC')
    
    ax.append(f.add_subplot(gs[1, 0]))
    
    shadedError(ax[1], snips1, linecolor=colors[0])
    shadedError(ax[1], snips2, linecolor=colors[1])
    
    ax[1].set_ylabel(ylabel)
    ax[1].spines['top'].set_visible(False)
    ax[1].spines['right'].set_visible(False)
    
    # ax[1].set_xticks([0, 10, 20, 30])
    # ax[1].set_xticklabels(['-10', '0', '10', '20'])
    ax[1].set_xlabel(xlabel)
    
    ax[1].text(xlen+labeloffset_x, np.mean(snips1, axis=0)[-1]-labeloffset_y, labels[0], color=colors[0], ha='left', va='center')
    ax[1].text(xlen+labeloffset_x, np.mean(snips2, axis=0)[-1]+labeloffset_y, labels[1], color=colors[1], ha='left', va='center')
    
    ax[0].set_xlim(ax[1].get_xlim())
    
    return f, ax


# -*- coding: utf-8 -*-
"""
Created on Fri Apr 17 10:14:34 2020

@author: James Edgar McCutcheon
"""

import csv
import xlrd

def metafilemaker(xlfile, metafilename, sheetname='metafile', fileformat='csv'):
    """
    Makes metafile (txt or csv) from Excel sheet for reading in by metafilereader

    Parameters
    ----------
    xlfile : String
        Path to Excel file to be opened containing sheet to be made into metafile.
    metafilename : String 
        Path for metafile to be saved into (without extension).
    sheetname : String, optional
        Name of sheet in Excel file. The default is 'metafile'.
    fileformat : String, optional
        File format to save in. Options are 'csv' or 'txt. The default is 'csv'.

    Returns
    -------
    None.

    """
    with xlrd.open_workbook(xlfile) as wb:
        sh = wb.sheet_by_name(sheetname)  # or wb.sheet_by_name('name_of_the_sheet_here')
        
        if fileformat == 'csv':
            with open(metafilename+'.csv', 'w', newline="") as f:
                c = csv.writer(f)
                for r in range(sh.nrows):
                    c.writerow(sh.row_values(r))
        if fileformat == 'txt':
            with open(metafilename+'.txt', 'w', newline="") as f:
                c = csv.writer(f, delimiter="\t")
                for r in range(sh.nrows):
                    c.writerow(sh.row_values(r))
    
def metafilereader(filename, sheetname="metafile"):
    """
    Reads in metafile. If an Excel file is given it uses the sheetname argument
    to specify sheet. Otherwise, text files work better than CSV files.

    Parameters
    ----------
    filename : String
        Path to metafile to be read in and interpreted.
    sheetname : String
        Name of sheet within excel file to be used. Default is metafile.

    Returns
    -------
    tablerows : List of lists
        Rows from metafile.
    header : List
        First row from metafile which provides column names.

    """
    
    if filename.lower().endswith((".xls", "xlsx")):
        print("File is excel file. Making csv metafile first")
        
        with xlrd.open_workbook(filename) as wb:
            sh = wb.sheet_by_name(sheetname)  # or wb.sheet_by_name('name_of_the_sheet_here')
            
            header = sh.row_values(0)
            
            tablerows=[]
            for r in range(1, sh.nrows):
                tablerows.append(sh.row_values(r))
                
        return tablerows, header

    f = open(filename, 'r')
    f.seek(0)
    header = f.readlines()[0]
    f.seek(0)
    filerows = f.readlines()[1:]
    
    tablerows = []
    
    for i in filerows:
        tablerows.append(i.split('\t'))
        
    header = header.split('\t')
    # need to find a way to strip end of line \n from last column - work-around is to add extra dummy column at end of metafile
    return tablerows, header


# -*- coding: utf-8 -*-
"""
Created on Fri Apr 17 11:51:29 2020

@author: James Edgar McCutcheon
"""

import numpy as np
import scipy.signal as sig
from math import ceil

def processdata(data, datauv, method='konanur', normalize=True, normalize_time_cutoff=5, normalize_method="zscore", fs=1017):
    """ Corrects for baseline when given calcium-moldulated and non-Ca modulated streams.

    Parameters
    ----------
    data : List or 1D array of Floats
        Primary data stream (Ca-modulated).
    datauv : List or 1D array of Floats
        Secondary data stream (non-Ca modulated).
    method : Str, optional
        Chooses method to correct. Options are 'konanur', 'lerner-davidson', 'lerner') The default is 'konanur'.
        'konanur' was developed by Vaibhav Konanur and uses FFT as described in
        doi: 10.1016/j.physbeh.2019.112771
        
        'lerner-davidson' (aka lerner or davidson) uses regression and was developed
        by Tom Davidson and used by Talia Lerner as described in
        doi: 10.1016/j.cell.2015.07.014
        
        Code adapted from https://github.com/tjd2002/tjd-shared-code/tree/master/matlab/photometry/FP_normalize.m
        
    normalize : Bool, optional
        Normalizes signal by method given in normalize_method. The default is True.
    normalize_time_cutoff : Int, optional
        Time in minutes at beginning and end to ignore when normalizing signal.
        If too long (> a quarter of total time) then it is reduced. The default is 5 (min).
    normalize_method : Str, optional
        Default is zscore. But can also use old method which uses 3*SD.
    fs : Int, optional
        Used when normalizing signal. 1017 is default.

    Returns
    -------
    df : List or 1D array of Floats
        Processed signal, corrected for isosbestic.

    """
    if method == 'konanur':
        pt = len(data)
        X = np.fft.rfft(datauv, pt)
        Y = np.fft.rfft(data, pt)
        Ynet = Y-X
    
        df = np.fft.irfft(Ynet) 
        df = sig.detrend(df)
    
        b, a = sig.butter(9, 0.012, 'low', analog=True)
        df = sig.filtfilt(b, a, df)
        
    elif method == 'lerner' or method == 'lerner-davidson' or method == 'davidson':
        x = np.array(datauv)
        y = np.array(data)
        bls = np.polyfit(x, y, 1)
        Y_fit_all = np.multiply(bls[0], x) + bls[1]
        Y_dF_all = y - Y_fit_all
        df = np.multiply(100, np.divide(Y_dF_all, Y_fit_all))
        
    else:
        print(method, 'is not a valid method.')
        
    if normalize==True:
        fs = int(fs)
        total_length = len(df) / (60*fs)

        if total_length < normalize_time_cutoff*2:
            nsamples = int((total_length / 4) * 60 * fs)
            print("File is too short ({:.1f} minutes) for {} minute cutoff. Using {:.1f} minutes instead.".format(
                total_length, normalize_time_cutoff, nsamples / (60*fs)))
        else:
            nsamples = normalize_time_cutoff*60*fs
            
        cutoff_range = range(nsamples, len(df)-nsamples)
        mean=abs(np.mean(df[cutoff_range]))
        sd=np.std(df[cutoff_range])
        
        if normalize_method == "zscore":
            df_corr = np.subtract(df, mean)
            df=np.divide(df_corr, sd)
        elif normalize_method == "old":
            df=np.divide(df, sd*3)
    
    return df

def snipper(data, timestamps, fs=1, baseline_length=10, trial_length=30,
                 adjust_baseline = True,
                 bins = 0, **kwargs):
    """ Makes 'snips' of a data file aligned to an event of interest.

    If a timelocked map is needed to align data precisely (e.g. with TDT equipment)
    then it is necessary to pass a t2sMap to the function.

    Parameters
    ----------
    data : List or array of floats
        Data to be divided into snips.
    timestamps : List
        Timestamps of events to be used to align data.
    fs : Float, optional
        Sampling frequency. The default is 1.
    pre_trial : Int, optional
        Number of seconds to include before event. The default is 10.
    trial_length : Int, optional
        Total length (in seconds) of each snip. The default is 30.
    adjust_baseline : Bool, optional
        When True divides all data points in each snip by mean of points occurring before event. The default is True.
    bins : Int, optional
        Number of bins to divide trial length into. The default is 0. If set at default
        of 0 then no binning will occur.

    Returns
    -------
    snips : List of lists
        List of X snips of Y length where X=number of events in timelock and Y=bins
    pps : Int
        Samples (points) per second 

    """
    # legacy arguments
    if "timelock " in kwargs.keys():
        timestamps = kwargs["timelock"]
    
    if "preTrial" in kwargs.keys():
        baseline_length = kwargs["preTrial"]

    if "trialLength" in kwargs.keys():
        trial_length = kwargs["trialLength"]
    
    if "adjustBaseline" in kwargs.keys():
        adjust_baseline = kwargs["adjustBaseline"]

    if 't2sMap' in kwargs.keys():
        print("Use of t2sMap is no longer supported. Use fs instead or install trompy=<14.0")
        return
    
    if len(timestamps) == 0:
        print('No events to analyse! Quitting function.')
        raise Exception('no events')
    
    # removes non-numeric values, e.g. nans or infinite
    timestamps = [i for i in timestamps if np.isfinite(i)]
    
    events_in_samples = [ceil(timestamp*fs) for timestamp in timestamps]

    baseline_in_samples = int(baseline_length*fs)
    trial_length_in_samples = int(trial_length*fs)

    # removes events where an entire snip cannot be made
    events_in_samples = [event for event in events_in_samples if \
        (event-baseline_in_samples > 0) and \
        (event-baseline_in_samples+trial_length_in_samples) < len(data)]

    n_snips = len(events_in_samples)
    snips = np.empty([n_snips, trial_length_in_samples])
    
    trial_start = [event - baseline_in_samples for event in events_in_samples]
    trial_end = [start + trial_length_in_samples for start in trial_start]

    for idx, (start, end) in enumerate(zip(trial_start, trial_end)):
        snips[idx] = data[start : end]

    if adjust_baseline == True:
        average_baseline = np.mean(snips[:,:baseline_in_samples], axis=1)
        snips = np.subtract(snips.transpose(), average_baseline).transpose()
        # snips = np.divide(snips.transpose(), np.abs(average_baseline)).transpose()

    if bins > 0:
        if trial_length_in_samples % bins != 0:
            snips = snips[:,:-(trial_length_in_samples % bins)]
        totaltime = snips.shape[1] / int(fs)
        snips = np.mean(snips.reshape(n_snips,bins,-1), axis=2)
        pps = bins/totaltime
              
    return snips, int(fs)

def mastersnipper(data, dataUV, data_filt, fs, events,
                  trialLength=30,
                  snipfs=10,
                  preTrial=10,
                  threshold=8,
                  peak_between_time=[0, 1],
                  latency_events=[],
                  latency_direction='pre',
                  max_latency=30,
                  verbose=True,
                  removenoisefromaverage=True,
                  **kwargs):

    """ Runs snipper function on several types of data streams and calculates noise trials.

    Parameters
    ----------
    data : List or 1D array
        Primary data stream (normally from blue channel).
    dataUV : List or 1D array
        Secondary data stream (normally from UV channel).
    data_filt : List or 1D array
        Filtered data stream (normally after Lerner or Vaibhav correction of data and dataUV).
    t2sMap : List
        Time-to-sample map.
    fs : Float
        Sample frequency.
    events : List of floats
        Timestamps of events for data to be aligned to.
    bins : Int, optional
        Number of bins for snips to be in length. The default is 300.
    baselinebins : Int, optional
        Number of bins for baseline. The default is 100.
    preTrial : Int, optional
        Time (in seconds) before event. The default is 10.
    trialLength : Int, optional
        Time (in seconds) for entire snip/trial. The default is 30.
    threshold : Int or Float, optional
        Noise threshold factor - snips that exceed threshold x bgMAD are considered 'noisy'. The default is 8.
    peak_between_time : List, optional
        Start and stop times for peak calculation. The default is [0, 1].
    latency_events : List, optional
        Timestamps for latency events. The default is [].
    latency_direction : String, optional
        Either 'pre' or 'post' depedning on whether to search for first latency before or after alignment event. The default is 'pre'.
    max_latency : Int or Float, optional
        Amount of time before or after alignment event to search for a latency event. The default is 30.
    verbose : Bool, optional
        Prints out information as it analyzes. The default is True.

    Returns
    -------
    output, a dictionary with the following keys
    'blue' : 2D array (or list of lists of floats)
        Snips of primary data stream.
    'uv' : 2D array (or list of lists of floats)
        Snips of secondary data stream.
    'filt' : 2D array (or list of lists of floats)
        Snips of filtered data stream.    
    'filt_z' : 2D array (or list of lists of floats)
        Snips of Z-scored filtered data stream.         
    'filt_avg' : 1d array (or list of floats)
        Average (mean) of filtered snips.
    'filt_avg_z' : 1d array (or list of floats)
        Average (mean) of filtered snips converted to z-score.
    'noise' : List of Booleans    
        Logical index of noisy trials.
    'peak' : List of floats
        Peak values for all snips.
    'latency' : List of floats
        Latency times for all trials.

    """
    # Parse arguments relating to trial length, bins etc

    if preTrial > trialLength:
        baseline = trialLength / 2
        print("preTrial is too long relative to total trialLength. Changing to half of trialLength.")
    else:
        baseline = preTrial
    
    if 'bins' in kwargs.keys():
        bins = kwargs['bins']
        print(f"bins given as kwarg. Fs for snip will be {trialLength/bins} Hz.")
    else:
        if snipfs > fs-1:
            print('Snip fs is too high, reducing to data fs')
            bins = 0
        else:
            bins = int(trialLength * snipfs)
    
    print('Number of bins is:', bins)
    
    if 'baselinebins' in kwargs.keys():
        baselinebins = kwargs['baselinebins']
        if baselinebins > bins:
            print('Too many baseline bins for length of trial. Changing to length of baseline.')
            baselinebins = int(baseline*snipfs)
        baselinebins = baselinebins
    else:
        baselinebins = int(baseline*snipfs)
    
    if len(events) < 1:
        print('Cannot find any events. All outputs will be empty.')
        return {}
    else:
        if verbose: print('{} events to analyze.'.format(len(events)))

    if len(data) > 0:
        blueTrials,_ = snipper(data, events,
                                       fs=fs,
                                       bins=bins,
                                       preTrial=baseline,
                                       trialLength=trialLength)
    else:
        print('No data stream available as primary data input. Exiting without snipping.')
        return {}
    
    if len(dataUV) > 0:
        uvTrials,_ = snipper(dataUV, events,
                                   fs=fs,
                                   bins=bins,
                                   preTrial=baseline,
                                   trialLength=trialLength)
    else:
        print('No UV (secondary) data stream.')
        uvTrials = []
    if len(data_filt) > 0:
        filtTrials,_ = snipper(data_filt, events,
                                   fs=fs,
                                   bins=bins,
                                   preTrial=baseline,
                                   trialLength=trialLength,
                                   adjustBaseline=False)

        filtTrials_z = np.asarray(zscore(filtTrials, baseline_points=baselinebins))
    else:
        print('No processed data stream.')
        filtTrials, filtTrials_z  = [], []

    # Code to calculate noise in file and work out which trials should be classified as noisy
    if 'bgMAD' not in kwargs.keys():
        bgMAD = findnoise(data, makerandomevents(120, int(len(data)/fs)-120),
                              fs=fs, bins=bins,
                              method='sum')
    else:
        bgMAD = kwargs['bgMAD']

    sigSum = [np.sum(abs(i)) for i in blueTrials]
    noiseindex = [i > bgMAD*threshold for i in sigSum]

    if sum(noiseindex) == len(noiseindex):
        print('Not removing noisy trials from average because all trials identified as noisy.')
        removenoisefromaverage = False
    if len(filtTrials) > 0:
        if removenoisefromaverage:
                filt_avg = np.mean(removenoise(filtTrials, noiseindex), axis=0)
                if verbose: print('{} noise trials removed from averages'.format(sum(noiseindex)))
        else:
            filt_avg = np.mean(filtTrials, axis=0)
            
        filt_avg_z = [(x-np.mean(filt_avg))/np.std(filt_avg) for x in filt_avg]
    else:
        filt_avg, filt_avg_z  = [], []
    
    # Code to find peak at specified time in each trial
    bin2s = bins/trialLength
    peakbins = [int((preTrial+peak_between_time[0])*bin2s),
                int((preTrial+peak_between_time[1])*bin2s)]
    peak = [np.mean(trial[peakbins[0]:peakbins[1]]) for trial in filtTrials_z]
    
    # Code to find latencies associated with each trial
    latency = []
    if len(latency_events) > 1: 
        for event in events:
            if latency_direction == 'pre':
                try:
                    latency.append(np.abs([lat-event for lat in latency_events if lat-event<0]).min())
                except ValueError:
                    latency.append(np.NaN)
            
            elif latency_direction == 'post':
                try:
                    latency.append(np.abs([lat-event for lat in latency_events if lat-event>0]).min())
                except ValueError:
                    latency.append(np.NaN)

        latency = [x if (x<max_latency) else np.NaN for x in latency]
        if latency_direction == 'pre':
            latency = [-x for x in latency]
    else:
        print('No latency events found')
        
    # Create info dictionary
    info = {}
    info['baseline'] = baseline
    info['length'] = trialLength
    if bins == 0:
        info['bins'] = np.shape(blueTrials)[1]
    else:
        info['bins'] = bins
    info['snipfs'] = info['bins'] / info['length']

    output = {}
    output['blue'] = blueTrials
    output['uv'] = uvTrials
    output['filt'] = filtTrials
    output['filt_z'] = filtTrials_z
    output['filt_avg'] = filt_avg
    output['filt_avg_z'] = filt_avg_z
    output['noise'] = noiseindex
    output['peak'] = peak
    output['latency'] = latency
    output['info'] = info
    
    return output

def zscore(snips, baseline_points=100):
    """
    Converts list or array of snips into Z-scored values.

    Parameters
    ----------
    snips : List of lists or 2D Array
        Data to be converted into z-scores.
    baseline_points : Int, optional
        Number of bins or points to be used as the baseline for z-score calculation. The default is 100.

    Returns
    -------
    z_snips : List of lists
        Converted snips expressed as z-scores.
    """
    
    BL_range = range(baseline_points)
    z_snips = []
    for i in snips:
        mean = np.mean(i[BL_range])
        sd = np.std(i[BL_range])
        z_snips.append([(x-mean)/sd for x in i])
        
    return z_snips

def findnoise(data, background_events, fs = 1, bins=0, method='sd'):
    """
    Identifies snips that are classed as noisy due to exceeding a threshold based on background.

    Parameters
    ----------
    data : List or 1D array of Floats
        Data stream from entire session.
    background_events : List of floats
        Timestamps from which background snips will be made.
    t2sMap : List of floats, optional
        Time-to-samples map. The default is [].
    fs : Float, optional
        Sampling frequency. The default is 1.
    bins : Int, optional
        Number of bins to be used for snips. The default is 0.
    method : Str, optional
        Method of calculating noise. 'sd' or 'sum' (standard deviation or sum). Default is 'sd'.

    Returns
    -------
    bgMAD : Float
        Median absoluate deviation of background trials.

    """
    
    bgSnips, _ = snipper(data, background_events, fs=fs, bins=bins)
    
    if method == 'sum':
        bgSum = [np.sum(abs(i)) for i in bgSnips]
        bgMAD = med_abs_dev(bgSum)
    elif method == 'sd':
        bgSD = [np.std(i) for i in bgSnips]
        bgMAD = med_abs_dev(bgSD)
   
    return bgMAD

def removenoise(snipsIn, noiseindex):
    """
    Removes snips that have been classified as noisy.

    Parameters
    ----------
    snipsIn : List of lists or Numpy array
        Snips (generally list of binned data).
    noiseindex : List of Booleans
        Should be the same length as first dimension of snipsIn.

    Returns
    -------
    snipsOut : List of lists
        Snips with noisy snips removed.

    """
    snipsOut = np.array([x for (x,v) in zip(snipsIn, noiseindex) if not v])   
    return snipsOut

def med_abs_dev(data, b=1.4826):
    """
    Calcuates median absolute deviation. See doi:10.1016/j.jesp.2013.03.013

    Parameters
    ----------
    data : List or 1D array
        Data to be analyzed.
    b : Float, optional
        Constant for calcuation. The default is 1.4826.

    Returns
    -------
    mad : Float
        Median absolute deviation of data.

    """
    median = np.median(data)
    devs = [abs(i-median) for i in data]
    mad = np.median(devs)*b
                   
    return mad

def makerandomevents(minTime, maxTime, spacing = 77, n=100):
    """
    Generates pseudorandom event timestamps in between a start and stop time.
    Rather than using purely random selection should ensure events are tiled.

    Parameters
    ----------
    minTime : Int
        Minimum time (in seconds).
    maxTime : Int
        Maximum time (in seconds).
    spacing : Int, optional
        Time (in seconds) between consecutive events. Provides pseudorandom timing. The default is 77.
    n : Int, optional
        Number of timestamps to generate. The default is 100.

    Returns
    -------
    events : List
        Pseudorandonm timestamps.

    """
    events = []
    total = maxTime-minTime
    start = 0
    for i in np.arange(0,n):
        if start > total:
            start = start - total
        events.append(start)
        start = start + spacing
    events = [i+minTime for i in events]
    return events

def time2samples(data, fs):
    """
    Makes time2samples map (t2sMap) used to convert from timestamps to sample number.
    
    Edited 20-04-2020 - removed Tick as necessary argument.

    Parameters
    ----------
    data : List or 1D array
        Streamed photometry data.
    fs : Float
        Sample frequency.

    Returns
    -------
    t2sMap : 1D array
        Array of time (in second) for  number of samples in data.

    """
    t2sMap = np.linspace(1, len(data), len(data)) / fs
    
    # maxsamples = len(tick)*int(fs)
    # if (len(data) - maxsamples) > 2*int(fs):
    #     print('Something may be wrong with conversion from time to samples')
    #     print(str(len(data) - maxsamples) + ' samples left over. This is more than double fs.')
    #     t2sMap = np.linspace(min(tick), max(tick), maxsamples)
    # else:
    #     t2sMap = np.linspace(min(tick), max(tick), maxsamples)
        
    return t2sMap
    
def event2sample(EOI, t2sMap):
    """
    Returns sample number for an event.

    Parameters
    ----------
    EOI : Float
        Timestamp of an event (in seconds).
        
    t2sMap : 1D array
        Array of times to convert to sample number

    Returns
    -------
    idx : Int
        Index for event of interest in t2sMap, e.g. sample number.

    """
    idx = (np.abs(t2sMap - EOI)).argmin()   
    return idx

def resample_snips(snips, factor=0.1):
    """ Resamples snips to collapse data into larger bins (e.g. for ROC analysis)
    
    Parameters
    ------------
    snips : 2D array or list of lists of floats
        Snips to be resampled.
    factor : Float, optional
        Constant to determine how to bin data. Default is 0.1.
    
    Returns
    snips : 2D array or list of lists of floats
        Resampled snips.
    
    """
    if len(snips)>0:
        n_bins = len(snips[0])
        out_bins = int(n_bins * factor)

        snips_out = []
        for snip in snips:
            snips_out.append(np.mean(np.reshape(snip, (out_bins, -1)), axis=1))
    
        return snips_out
    else:
        return []


# -*- coding: utf-8 -*-
"""
Created on Sun Mar 20 13:11:39 2022

@author: jmc010
"""
import sys
import getopt
import pandas as pd

def merge_files(files, output):
    print("merging files...", files)

    dfs = []
    for file in files:
        dfs.append(pd.read_csv(file))
    
    df_out = pd.concat(dfs)
    
    print("creating file...", output)
    
    df_out.to_csv(output, index=False)
    
def parse_args(argv):
    arg_input = ""
    arg_output = ""
    arg_help = "{0} -i <input> -o <output>".format(argv[0])
    
    try:
        opts, args = getopt.getopt(argv[1:], "hi:o:", ["help", "input=", "output="])
    except:
        print(arg_help)
        sys.exit(2)
    
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            print(arg_help)  # print the help message
            sys.exit(2)
        elif opt in ("-i", "--input"):
            arg_input = arg
        elif opt in ("-o", "--output"):
            arg_output = arg

    print('input:', arg_input)
    print('output:', arg_output)
    
    files = arg_input.split(",")
    files = [file.strip() for file in files]

    merge_files(files, arg_output)

if __name__ == "__main__":
    parse_args(sys.argv)


# -*- coding: utf-8 -*-
"""
Created on Fri Apr 17 10:12:10 2020

@author: James Edgar McCutcheon
"""

import numpy as np
import string
import datetime

def medfilereader(filename, varsToExtract = 'all',
                  sessionToExtract = 1,
                  verbose = False,
                  remove_var_header = False):
    
    """Reads in Med Associates file stored as single column and returns variables as lists.
    
    Parameters
    ----------
    filename : str
        File to be read in.
    varsToExtract : str or list of str, optional
        (e.g. ['a', 'b', 'f']), default is 'all'
    sessionToExtract : int, optional
        Can be specified for situations in which more than one session is included in a single file. Deafult is 1.
    verbose : bool, optional
        Prints statements with file information. Default is False.
    remove_var_header : bool, optional
        Removes first value in array, useful when negative numbers are used as markers to signal array start. Default is False.
    
    Returns
    -------
    varsToReturn : list of lists of numbers (int or float)
        Variables extracted from medfile as lists or a list of lists ('all')
    """

    if varsToExtract == 'all':
        numVarsToExtract = np.arange(0,26)
    else:
        numVarsToExtract = [ord(x)-97 for x in varsToExtract]
    
    f = open(filename, 'r')
    f.seek(0)
    filerows = f.readlines()[8:]
    datarows = [isnumeric(x) for x in filerows]
    matches = [i for i,x in enumerate(datarows) if x == 0.3]
    if sessionToExtract > len(matches):
        print('Session ' + str(sessionToExtract) + ' does not exist.')
    if verbose == True:
        print('There are ' + str(len(matches)) + ' sessions in ' + filename)
        print('Analyzing session ' + str(sessionToExtract))
    
    varstart = matches[sessionToExtract - 1]
    medvars = [[] for n in range(26)]
    
    k = int(varstart + 27)
    for i in range(26):
        medvarsN = int(datarows[varstart + i + 1])
        
        medvars[i] = datarows[k:k + int(medvarsN)]
        k = k + medvarsN
        
    if remove_var_header == True:
        varsToReturn = [medvars[i][1:] for i in numVarsToExtract]
    else:
        varsToReturn = [medvars[i] for i in numVarsToExtract]

    if np.shape(varsToReturn)[0] == 1:
        varsToReturn = varsToReturn[0]
    return varsToReturn

def isnumeric(s):
    """Converts strings into numbers (floats)

    Parameters
    ----------
    s : any
    
    Returns
    -------
    x : float
    """
    
    try:
        x = float(s)
        return x
    except ValueError:
        return float('nan')
    
def checknsessions(filename):
    '''Helper function for medfilereader that checks how many sessions in medfile'''
    
    f = open(filename, 'r')
    f.seek(0)
    filerows = f.readlines()[8:]
    datarows = [isnumeric(x) for x in filerows]
    matches = [i for i,x in enumerate(datarows) if x == 0.3]
    return matches

def tstamp_to_tdate(timestamp, fmt):
    '''Converts timestamp in string format into datetime object'''
    
    try:
        return datetime.datetime.strptime(timestamp, fmt)
    except ValueError:
        return

def medfilereader_licks(filename,
                  sessionToExtract = 1,
                  verbose = False,
                  remove_var_header = True):
    
    """Reads in Med Associates file stored as single column and returns variables as lists.
    
    Parameters
    ----------
    filename : str
        File to be read in.
    sessionToExtract : int, optional
        Can be specified for situations in which more than one session is included in a single file. Deafult is 1.
    verbose : bool, optional
        Prints statements with file information. Default is False.
    remove_var_header : bool, optional
        Removes first value in array, useful when negative numbers are used as markers to signal array start. Default is False.
    
    Returns
    -------
    medvars : list of lists of numbers (int or float)
        Variables extracted from medfile as lists or a list of lists ('all')
    """
    
    f = open(filename, 'r')
    f.seek(0)
    filerows = f.readlines()[8:]
    datarows = [isnumeric(x) for x in filerows]
    matches = [i for i,x in enumerate(datarows) if x == 0.3]
    if sessionToExtract > len(matches):
        print('Session ' + str(sessionToExtract) + ' does not exist.')
    if verbose == True:
        print('There are ' + str(len(matches)) + ' sessions in ' + filename)
        print('Analyzing session ' + str(sessionToExtract))
    
    varstart = matches[sessionToExtract - 1]    
    medvars = {}
   
    k = int(varstart + 27)
    for i in range(26):
        medvarsN = int(datarows[varstart + i + 1])
        if medvarsN > 1:
            medvars[string.ascii_uppercase[i]] = datarows[k:k + int(medvarsN)]
        k = k + medvarsN
    
    if remove_var_header == True:
        for val in medvars.values():
            val.pop(0)

    return medvars


# -*- coding: utf-8 -*-
"""
Created on Fri Apr 17 16:23:32 2020

@author: James Edgar McCutcheon
"""
import matplotlib.pyplot as plt
import numpy as np

def trialsFig(ax, trials, pps=1, preTrial=10, scale=5, noiseindex = [],
              plotnoise=True,
              eventText='event', 
              ylabel=''):
    """ Plots single trials and mean (e.g. photometry signaL timelocked to an event).
    
    Parameters
    -----------
    ax : Matplotlib axis object
        Axis where data will be plotted.
    trials : List of list of floats or 2D array
        Data to be plotted where rows are individual trials and columns are time bins.
    pps : Int, optional
        Points per second. Default is 1.
    preTrial : Int or Float, optional
        Time (generally in seconds) preceding timelocked event. Default is 10.
    scale : Int, optional
        Scale bar length in seconds, Default is 5.
    noiseindex : List of Booleans, optional
        Can provide indices of trials that exceed a certain threshold for exclusion. Deafult is [].
    plotnoise : Bool, optional
        If False, will exclude any trials provided as indices in noise index. Default is True.
    eventText : Str, optional
        Text on graph can be customized, e.g. Pellet. Default is "event".
    ylabel : Str, optional
        Text for y-axis label. Default is "".
    
    Returns
    --------
    ax : Matplotlib axis object
        Axis object is returned. 
    """

    if len(noiseindex) > 0:
        trialsNoise = np.array([i for (i,v) in zip(trials, noiseindex) if v])
        trials = np.array([i for (i,v) in zip(trials, noiseindex) if not v])
        if plotnoise == True:
            ax.plot(trialsNoise.transpose(), c='red', alpha=0.1)
        
    ax.plot(trials.transpose(), c='grey', alpha=0.4)
    ax.plot(np.mean(trials,axis=0), c='k', linewidth=2)
     
    ax.set(ylabel = ylabel)
    ax.xaxis.set_visible(False)
            
    scalebar = scale * pps

    yrange = ax.get_ylim()[1] - ax.get_ylim()[0]
    scalebary = (yrange / 10) + ax.get_ylim()[0]
    scalebarx = [ax.get_xlim()[1] - scalebar, ax.get_xlim()[1]]
    
    ax.plot(scalebarx, [scalebary, scalebary], c='k', linewidth=2)
    ax.text((scalebarx[0] + (scalebar/2)), scalebary-(yrange/50), str(scale) +' s', ha='center',va='top')
 
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.spines['bottom'].set_visible(False)
    
    xevent = pps * preTrial  
    ax.plot([xevent, xevent],[ax.get_ylim()[0], ax.get_ylim()[1] - yrange/20],'--')
    ax.text(xevent, ax.get_ylim()[1], eventText, ha='center',va='bottom')
    
    return ax

def trialsMultFig(ax, trials, pps=1, preTrial=10, scale=5,
              eventText='event', 
              ylabel='',
              linecolor=['m', 'b'],):
    """ Plots single trials from multiple datasets (e.g. photometry signaL timelocked to an event).
    
    Parameters
    -----------
    ax : Matplotlib axis object
        Axis where data will be plotted.
    trials : Tuple of list of list of floats or 3D array
        Data to be plotted of shape (2,x,y) where first dimension is different datasets.
    pps : Int, optional
        Points per second. Default is 1.
    preTrial : Int or Float, optional
        Time (generally in seconds) preceding timelocked event. Default is 10.
    scale : Int, optional
        Scale bar length in seconds, Default is 5.
    ylabel : Str, optional
        Text for y-axis label. Default is "".
    linecolor : List of two strings, optional
        Colors to use for plotting different datasets. Default is ['m', 'b'] (magenta and blue).
    
    Returns
    --------
    ax : Matplotlib axis object
        Axis object is returned.
        
    Notes
    ------
    Not used very often. May need to be revised to be more functional.
    """    
    for i in [0, 1]:
        y = trials[i].transpose()   
        ax.plot(y, c=linecolor[i], linewidth=1)
     
    ax.set(ylabel = ylabel)
    ax.xaxis.set_visible(False)
            
    scalebar = scale * pps

    yrange = ax.get_ylim()[1] - ax.get_ylim()[0]
    scalebary = (yrange / 10) + ax.get_ylim()[0]
    scalebarx = [ax.get_xlim()[1] - scalebar, ax.get_xlim()[1]]
    
    ax.plot(scalebarx, [scalebary, scalebary], c='k', linewidth=2)
    ax.text((scalebarx[0] + (scalebar/2)), scalebary-(yrange/50), str(scale) +' s', ha='center',va='top')
 
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.spines['bottom'].set_visible(False)
    
    xevent = pps * preTrial
    ax.plot([xevent, xevent],[ax.get_ylim()[0], ax.get_ylim()[1] - yrange/20],'--')
    ax.text(xevent, ax.get_ylim()[1], eventText, ha='center',va='bottom')
    
    return ax

def trialsShadedFig(ax, trials, pps = 1, scale = 5, preTrial = 10,
                    noiseindex=[],
                    eventText = 'event', ylabel = '',
                    linecolor='k', errorcolor='grey'):
    
    """ Plots multiple trials as a shaded error plot (e.g. photometry signaL timelocked to an event).

    Parameters
    -----------
    ax : Matplotlib axis object
        Axis where data will be plotted.
    trials : List of list of floats or 2D array
        Data to be plotted where rows are individual trials and columns are time bins.
    pps : Int, optional
        Points per second. Default is 1.
    scale : Int, optional
        Scale bar length in seconds, Default is 5.
    preTrial : Int or Float, optional
        Time (generally in seconds) preceding timelocked event. Default is 10.
    noiseindex : List of Booleans, optional
        Can provide indices of trials that exceed a certain threshold for exclusion. Deafult is [].
    eventText : Str, optional
        Text on graph can be customized, e.g. Pellet. Default is "event".
    ylabel : Str, optional
        Text for y-axis label. Default is "".
    linecolor : Str, optional
        Color to use for plotting mean. Default is "k" (black).
    errorcolor : Str, optional
        Color to plot shaded error in. Default is "grey".

    Returns
    --------
    ax : Matplotlib axis object
        Axis object is returned.
    """
    
    if len(noiseindex) > 0:
        trials = np.array([i for (i,v) in zip(trials, noiseindex) if not v])
    
    yerror = [np.std(i)/np.sqrt(len(i)) for i in trials.T]
    y = np.mean(trials,axis=0)
    x = np.arange(0,len(y))
    
    ax.plot(x, y, c=linecolor, linewidth=2)
    
    errorpatch = ax.fill_between(x, y-yerror, y+yerror, color=errorcolor, alpha=0.4)
    
    ax.set(ylabel = ylabel)
    ax.xaxis.set_visible(False)
            
    scalebar = scale * pps
    
    yrange = ax.get_ylim()[1] - ax.get_ylim()[0]
    scalebary = (yrange / 10) + ax.get_ylim()[0]
    scalebarx = [ax.get_xlim()[1] - scalebar, ax.get_xlim()[1]]
    
    ax.plot(scalebarx, [scalebary, scalebary], c='k', linewidth=2)
    ax.text((scalebarx[0] + (scalebar/2)), scalebary-(yrange/50), '5 s', ha='center',va='top')
     
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.spines['bottom'].set_visible(False)
    
    xevent = pps * preTrial
    ax.plot([xevent, xevent],[ax.get_ylim()[0], ax.get_ylim()[1] - yrange/20],'--')
    ax.text(xevent, ax.get_ylim()[1], eventText, ha='center',va='bottom')
    
    return ax, errorpatch

def trialsMultShadedFig(ax, trials, pps = 1, scale = 5, preTrial = 10,
                        noiseindex=[],
                        eventText = 'event', ylabel = '',
                        linecolor=['m', 'b'], errorcolor=['r', 'g'],
                        title=''):
    """ Plots two different datasets as multiple trials with shaded error plot (e.g. photometry signaL timelocked to an event).
     
    Parameters
    -----------
    ax : Matplotlib axis object
        Axis where data will be plotted.
    trials : Tuple of list of list of floats or 3D array
        Data to be plotted of shape (2,x,y) where first dimension is different datasets.
    pps : Int, optional
        Points per second. Default is 1.
    scale : Int, optional
        Scale bar length in seconds, Default is 5.
    preTrial : Int or Float, optional
        Time (generally in seconds) preceding timelocked event. Default is 10.
    noiseindex : List of ints, optional
        Can provide indices of trials that exceed a certain threshold for exclusion. Deafult is [].
    eventText : Str, optional
        Text on graph can be customized, e.g. Pellet. Default is "event".
    ylabel : Str, optional
        Text for y-axis label. Default is "".
    linecolor : List of two strings, optional
        Colors to use for plotting means. Default is ['m', 'b'] (magenta and blue).
    errorcolor : List of two strings, optional
        Colors to plot shaded error in. Default is ['r', 'g'] (red and green).
    title : Str, optional
        Title for plot. Default is "".

    Returns
    --------
    ax : Matplotlib axis object
        Axis object is returned.
    """
    for i in [0, 1]:
        if len(noiseindex) > 0:
            trials[i] = np.array([i for (i,v) in zip(trials[i], noiseindex) if not v])
        yerror = [np.std(i)/np.sqrt(len(i)) for i in trials[i].T]
        y = np.mean(trials[i],axis=0)
        x = np.arange(0,len(y))

        ax.plot(x, y, c=linecolor[i], linewidth=2)

        errorpatch = ax.fill_between(x, y-yerror, y+yerror, color=errorcolor[i], alpha=0.4)
     
    ax.set(ylabel = ylabel)
    ax.xaxis.set_visible(False)
             
    scalebar = scale * pps
    
    yrange = ax.get_ylim()[1] - ax.get_ylim()[0]
    scalebary = (yrange / 10) + ax.get_ylim()[0]
    scalebarx = [ax.get_xlim()[1] - scalebar, ax.get_xlim()[1]]
     
    ax.plot(scalebarx, [scalebary, scalebary], c='k', linewidth=2)
    ax.text((scalebarx[0] + (scalebar/2)), scalebary-(yrange/50), '5 s', ha='center',va='top')

    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.spines['bottom'].set_visible(False)

    xevent = pps * preTrial
    ax.plot([xevent, xevent],[ax.get_ylim()[0], ax.get_ylim()[1] - yrange/20],'--')
    ax.text(xevent, ax.get_ylim()[1], eventText, ha='center',va='bottom')
    ax.set_title(title)

    return ax, errorpatch

def trialstiledFig(gs, trials, pps = 1, preTrial = 10):
    """ Plots individual trials as tiles. Good for searching for good representative figures.
    
    Parameters
    -----------
    gs : gridspec object
    trials : List of lists of floats or 2D array
    pps : Int, optional
    preTrial : Int
    
    Notes
    ------
    Not well tested. May need work.    
    """
    for i in np.arange(np.shape(trials)[0]):
        col=int(i/7)
        row=(i%7)+1

        ax = plt.subplot(gs[row, col])
        ax.plot(trials[i,:].transpose(), c='grey', alpha=0.4)   

        ax.spines['right'].set_visible(False)
        ax.spines['top'].set_visible(False)
        ax.spines['bottom'].set_visible(False)

def makeheatmap(ax, data, trial_to_plot="all", preTrial=10, pps=1, events=None, ylabel='Trial'):
    """ For making trial-by-trial heatmaps.
    
    Parameters
    -------------
    ax : Matplotlib axis object
        Axis to plot data in.
    data : List of list of floats or 2D array
        Data to be plotted where rows are individual trials and columns are time bins.
    trial_to_plot : Int or Str, optional
        Will place tick on specific trial if argument passed. Default is "all".
    preTrial : Int or Float, optional
        Time (generally in seconds) preceding timelocked event. Default is 10.
    pps : Int, optional
        Points per second. Default is 1.
    events : List of floats, optional
        Allows event times to be passed (e.g. latencies) which will be marked as white lines on heatplot.
    ylabel : Str, optional
        Label for y-axis. Default is "".
        
    Returns
    ---------
    ax : Matplotlib axis object
    mesh : Matplotlib mesh object
    
    """
    (ntrials, bins) = np.shape(data)

    xvals = np.linspace(-preTrial+(1/pps),(bins/pps)-preTrial,bins)
    yvals = np.arange(1, ntrials+2)
    xx, yy = np.meshgrid(xvals, yvals)
    
    mesh = ax.pcolormesh(xx, yy, data, cmap='jet', shading = 'auto')
    
    if events:
        ax.vlines(events, yvals[:-1], yvals[1:], color='w')
    else:
        print('No events')
        
    ax.set_ylabel(ylabel)
    if trial_to_plot == 'all':
        ax.set_yticks([1, ntrials])
    else:
        ax.set_yticks([1, trial_to_plot+1, ntrials])
    ax.set_xticks([])
    ax.invert_yaxis()
    
    return ax, mesh


# -*- coding: utf-8 -*-
# %##
"""
Created on Fri Apr 17 09:19:56 2020

@author: James Edgar McCutcheon
"""
import warnings
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from itertools import chain

import trompy as tp

class BarScatter():
    def __init__(self, data, linewidth, fontsize, bar_kwargs, sc_kwargs, ax_kwargs, extra_kwargs):
        self.data = np.asarray(data, dtype=object)

        self.linewidth = linewidth
        self.fontsize = fontsize

        self.bar_kwargs = bar_kwargs
        self.sc_kwargs = sc_kwargs
        self.ax_kwargs = ax_kwargs
        self.extra_kwargs = extra_kwargs

    def create_axis(self, ax):
        if isinstance(ax, plt.Axes):
            self.ax = ax
        else:
            f, self.ax = plt.subplots()

    def squeeze(self):
        if np.shape(self.data)[0] == 1:
            self.data = np.squeeze(self.data)
            self.squeeze()
        else:
            return

    def set_dims(self):
        # self.squeeze()
        self.grouped=False
        self.dims = np.ndim(self.data)
        self.n_groups = np.shape(self.data)[0]

        if self.dims == 1:
            try:
                self.data[0][0]
                self.dims = 2
            except:
                self.n_groups = 1
        elif self.dims == 2:
            try:
                self.data[0][0][0]
                self.grouped = True
                self.dims = 3
            except:
                pass
        elif self.dims == 3:
            self.grouped = True

    def prep_data(self, transpose=False):

        self.set_dims()

        data_obj = np.ndarray((np.shape(self.data)[:self.dims-1]), dtype=object)
        if self.grouped:
            for i1, dim1 in enumerate(self.data):
                for i2, dim2 in enumerate(dim1):
                    data_obj[i1][i2] = list(dim2)
            self.data = data_obj
        else:
            for i, dim in enumerate(self.data):
                data_obj[i] = list(dim)
            self.data = data_obj

        # Check if transpose = True
        if transpose:
            self.data = np.transpose(self.data)

    def calculate_items(self, barwidth, groupwidth):

        self.group_width = groupwidth
        self.group_x = np.arange(1,self.n_groups+1)
        self.bar_means = np.zeros((np.shape(self.data)[:self.dims-1]))
        self.bar_error = np.zeros((np.shape(self.data)[:self.dims-1]))

        if self.grouped:
            self.bars_per_group = np.shape(self.data)[1]
            self.width_of_bars = (barwidth * groupwidth) / self.bars_per_group
                
            for i in range(np.shape(self.data)[0]):
                for j in range(np.shape(self.data)[1]):
                    self.bar_means[i][j], self.bar_error[i][j] = tp.mean_and_sem(self.data[i][j])
        else:
            self.bars_per_group = 1
            self.width_of_bars = barwidth

            for i in range(np.shape(self.data)[0]):
                self.bar_means[i], self.bar_error[i] = tp.mean_and_sem(self.data[i])

    def calculate_x_vals(self):

        self.x_vals = np.zeros((np.shape(self.data)[:self.dims-1]))
        bar_allocation = self.group_width / self.bars_per_group
        k = (self.group_width/2) - (bar_allocation/2)
        
        if self.grouped == True:
            
            for i in range(np.shape(self.data)[0]):
                xrange = np.linspace(i+1-k, i+1+k, self.bars_per_group)
                for j in range(self.bars_per_group):
                    self.x_vals[i][j] = xrange[j]
        else:
            self.x_vals = self.group_x

    def set_colors(self, color_option, colors, paired_scatter = False):
        """ Helper function for setting colors in barscatter"""
            
        n_colors = len(colors)
        
        if (paired_scatter == True) & (color_option == 'within'):
            print('Not possible to make a Paired scatter plot with Within setting.')
            color_option = 'same'
            
        if color_option == 'within':
            if n_colors < self.bars_per_group:
                print('Not enough colors for this option! Reverting to one color.')
                color_option = 'same'
            elif n_colors > self.bars_per_group:
                colors = colors[:self.bars_per_group]
            color_output = [colors for i in self.data]
            color_output = list(chain(*color_output))
            
        if color_option == 'between':
            if n_colors < self.n_groups:
                print('Not enough colors for this option! Reverting to one color.')
                color_option = 'same'
            elif n_colors > self.n_groups:
                colors = colors[:self.n_groups]
            if paired_scatter == False:
                color_output = [[c]*self.bars_per_group for c in colors]
                color_output = list(chain(*color_output))
            else:
                color_output = colors
                
        if color_option == 'individual':
            if n_colors < self.n_groups*self.bars_per_group:
                print('Not enough colors for this color option')
                color_option = 'same'
            elif n_colors > self.n_groups*self.bars_per_group:
                color_output = colors[:self.n_groups * self.bars_per_group]
            else: 
                color_output = colors
        
        if color_option == 'same':
            color_output = [colors[0] for x in range(len(self.data.flatten()))]

        return color_output

    def xyspacer(self, x, y_vals, bindist=20, space=0.1):
        """ Helper function for barscatter for spacing individual datapoints"""
        
        histrange=[]
        histrange.append(min(self.ax.get_ylim()[0], min(y_vals)))
        histrange.append(max(self.ax.get_ylim()[1], max(y_vals)))

        yhist = np.histogram(y_vals, bins=bindist, range=histrange)
        
        self.x_vals=[]
        for ybin in yhist[0]:
            if ybin == 1:
                self.x_vals.append(x)
            elif ybin > 1:          
                temp_vals = np.linspace(x-space, x+space, num=ybin)
                for val in temp_vals:
                    self.x_vals.append(val)
                    
        self.y_vals = np.sort(self.y_vals)
    
    def make_bars(self, errorbars):
        self.barlist = []
        self.barx = []
        for x, y, e, bfc, bec in zip(self.x_vals.flatten(), self.bar_means.flatten(), self.bar_error.flatten(),
                                self.barfacecolorArray, self.baredgecolorArray):
            self.barx.append(x)
            if errorbars == True:
                self.bar_kwargs.update({"yerr": e})
            self.barlist.append(self.ax.bar(x, y, self.width_of_bars,
                            facecolor = bfc, edgecolor = bec,
                            zorder=-1,
                            linewidth=self.linewidth,
                            **self.bar_kwargs))
    
    def make_scatters(self, paired, spaced, yspace, xspace, scatterlinecolor, scattersize, scatteroffset):
        self.sclist = []
        self.x_vals =  self.x_vals + (scatteroffset*self.width_of_bars)/2
        
        if paired == False:
            for x, Yarray, scf, sce  in zip(self.x_vals.flatten(), self.data.flatten(),
                                            self.scfacecolorArray, self.scedgecolorArray):
                if spaced == True:
                    try: 
                        self.x_vals, self.y_vals = self.xyspacer(self.ax, x, Yarray, bindist=yspace, space=xspace)
                    except:
                        print("Could not space all sets of points.")
                        self.x_vals = [x] * len(Yarray)
                        self.y_vals = Yarray
                        
                    self.sclist.append(self.ax.scatter(self.x_vals, self.y_vals, s = scattersize,
                                c = scf,
                                edgecolors = sce,
                                **self.sc_kwargs))
                            
                else:
                    for y in Yarray:
                        self.sclist.append(self.ax.scatter(x, y, s = scattersize,
                                        c = scf,
                                        edgecolors = sce,
                                        **self.sc_kwargs))
                        
        elif self.grouped == True:
            for x, Yarray, scf, sce in zip(self.x_vals, self.data, self.scfacecolorArray, self.scedgecolorArray):
                for y in np.transpose(Yarray.tolist()):
                    self.sclist.append(self.ax.plot(x, y, '-o', markersize = scattersize/10,
                            color = scatterlinecolor,
                            markerfacecolor = scf,
                            markeredgecolor = sce,
                            markeredgewidth = self.sc_kwargs["linewidth"],
                            **self.sc_kwargs))
        elif self.grouped == False:
            for n,_ in enumerate(self.data[0]):
                y = [y[n-1] for y in self.data]
                self.sclist.append(self.ax.plot(self.x_vals, y, '-o', markersize = scattersize/10,
                            color = scatterlinecolor,
                            markerfacecolor = self.scfacecolorArray[0],
                            markeredgecolor = self.scedgecolorArray[0],
                            markeredgewidth = self.sc_kwargs["linewidth"],
                            **self.sc_kwargs))

    def set_axis_properties(self):


        self.ax.set(**self.ax_kwargs)
        self.xrange = np.diff(self.ax.get_xlim())
        self.yrange = np.diff(self.ax.get_ylim())

    def format_ticks(self):
        for axis in [self.ax.yaxis, self.ax.xaxis]:
            axis.set_tick_params(width=self.linewidth)
        self.ax.set_xticks([])
        self.ax.tick_params(labelbottom='off')
    
    def tidy_axes(self):
    
        self.ax.spines['right'].set_visible(False)
        self.ax.spines['top'].set_visible(False)
        self.ax.spines['bottom'].set_position('zero')

        self.ax.spines['bottom'].set_lw(self.linewidth)
        self.ax.spines['left'].set_lw(self.linewidth)
    
    def make_group_labels(self, grouplabel, grouplabeloffset):
        group_x = np.arange(1, len(grouplabel)+1)

        xpos = (group_x - self.ax.get_xlim()[0]) / self.xrange

        for x, label in zip(xpos, grouplabel):
            self.ax.text(x, -0.05+grouplabeloffset, label, va='top', ha='center', fontsize=self.fontsize, transform=self.ax.transAxes)

    def make_bar_labels(self, barlabels, barlabeloffset):
        xpos = (self.barx - self.ax.get_xlim()[0]) / self.xrange
        ypos = (-self.ax.get_ylim()[0]/self.yrange) - barlabeloffset

        for x, label in zip(xpos, barlabels):
            self.ax.text(x, ypos, label, va='top', ha='center', fontsize=self.fontsize, transform=self.ax.transAxes)

    def make_legend(self, itemlabels, legendloc):
        legendbar = []
        legendtext = []
        for i in range(self.bars_per_group):
            legendbar.append(self.barlist[i])
            legendtext.append(itemlabels[i])
        self.ax.legend(legendbar, legendtext, loc=legendloc)

    def process_kwargs(self):
        if "baralpha" in self.extra_kwargs.keys():
            self.bar_kwargs.update({"alpha": self.extra_kwargs["baralpha"]})

        if "scatteralpha" in self.extra_kwargs.keys():
            self.sc_kwargs.update({"alpha": self.extra_kwargs["scatteralpha"]})

        self.sc_kwargs.update({"linewidth": self.linewidth, "zorder": 20, "clip_on": False})

        legacy_ax_props = ["xlabel", "ylabel", "xlim", "ylim"]
        for prop in legacy_ax_props:
            if prop in self.extra_kwargs:
                self.ax_kwargs[prop] = self.extra_kwargs[prop]

        functional_kwargs = ["xlabel", "ylabel", "xlim", "ylim", "baralpha", "scatteralpha"]
        for key in self.extra_kwargs.keys():
            if key not in functional_kwargs:
                warnings.warn("{} is not a functional keyword argument. Check spelling or barscatter version.".format(key))

def barscatter(data_in, ax=[], transpose=False, paired=False,
               spaced=False, xspace=0.1, yspace=20,
               groupwidth = .75, barwidth = .8,
               barfacecolor_option="same", barfacecolor=["white"],
               baredgecolor_option="same", baredgecolor=["black"],
               errorbars=False, scatteroffset=0,
               scatterfacecolor_option="same", scatterfacecolor=["white"],
               scatteredgecolor_option="same", scatteredgecolor=["black"],
               scatterlinecolor="grey", linewidth=0.75, scattersize=80,
               grouplabel=[], grouplabeloffset=0,
               barlabels=[], barlabeloffset=0.025,
               itemlabels=[], show_legend=False, legendloc="upper right",
               fontsize=10,
               bar_kwargs={},
               sc_kwargs={},
               ax_kwargs={},
               **extra_kwargs):

    bs = BarScatter(data_in, linewidth, fontsize, bar_kwargs, sc_kwargs, ax_kwargs, extra_kwargs)

    bs.process_kwargs()

    bs.prep_data(transpose=transpose)
    bs.calculate_items(barwidth, groupwidth)
    bs.calculate_x_vals()

    bs.barfacecolorArray = bs.set_colors(barfacecolor_option, barfacecolor)
    bs.baredgecolorArray = bs.set_colors(baredgecolor_option, baredgecolor)

    bs.scfacecolorArray = bs.set_colors(scatterfacecolor_option, scatterfacecolor)
    bs.scedgecolorArray = bs.set_colors(scatteredgecolor_option, scatteredgecolor)

    bs.create_axis(ax)

    bs.make_bars(errorbars)

    bs.make_scatters(paired, spaced, xspace, yspace, scatterlinecolor, scattersize, scatteroffset)

    bs.set_axis_properties()
    bs.format_ticks()
    bs.tidy_axes()

    if len(grouplabel) == bs.n_groups:
        bs.make_group_labels(grouplabel, grouplabeloffset)
    
    if len(barlabels) == len(bs.barx):
        bs.make_bar_labels(barlabels, barlabeloffset)

    if show_legend:
        if len(itemlabels) == len(bs.barx):
            bs.make_legend(itemlabels, legendloc)
        elif len(barlabels) == len(bs.barx):
            bs.make_legend(barlabels, legendloc)
        else:
            barlabels=[str(x) for x in np.arange(1, len(bs.barx)+1)]
            bs.make_legend(barlabels, legendloc)
    
    return bs.ax, bs.barx, bs.barlist, bs.sclist


# #TODO change markerstyle
# #TODO read pandas series, dataframe etc
# #TODO add extra axis for estimation
# #TODO make sure fontsize is for all arguments
# #TODO make labels without offsets
# #TODO add tick kwargs
# #TODO catch exception for outer list or first dimension as 1, e.g. squeeze

# # def barscatter(data_in, transpose = False,
# #                 groupwidth = .75,
# #                 barwidth = .8,
# #                 paired = False,
# #                 unequal = False,
# #                 spaced = False,
# #                 yspace = 20,
# #                 xspace = 0.1,
# #                 barfacecolor_option = 'same', # other options 'between' or 'individual'
# #                 barfacecolor = ['white'],
# #                 baredgecolor_option = 'same',
# #                 baredgecolor = ['black'],
# #                 baralpha = 1,
# #                 scatterfacecolor_option = 'same',
# #                 scatterfacecolor = ['white'],
# #                 scatteredgecolor_option = 'same',
# #                 scatteredgecolor = ['grey'],
# #                 scatterlinecolor = 'grey', # Don't put this value in a list
# #                 scattersize = 80,
# #                 scatteralpha = 1,
# #                 spreadscatters = False,
# #                 linewidth=0.75,
# #                 xlim=[],
# #                 ylim=[],
# #                 ylabel = 'none',
# #                 xlabel = 'none',
# #                 grouplabel = [],
# #                 itemlabel = 'none',
# #                 barlabels = [],
# #                 barlabeloffset=0.025,
# #                 grouplabeloffset=0,
# #                 yaxisparams = 'auto',
# #                 show_legend = 'none',
# #                 legendloc='upper right',
# #                 xfontsize=8,
# #                 ax=[]):
# #     """
# #     Creates a bar graph with data points shown as overlaid circles.
    
# #     Parameters
# #     -------------
# #     data : List, lists of lists or array
# #         Data to be plotted. Will try to convert if needed.
# #     barwidth : Float, optional
# #         Width of bars. Default is 0.9.
# #     paired : Bool, optional
# #         Attempts to add lines between paired data points. Requires number in each group to be matched. Default is False.
# #     unequal : Bool, optional
# #         To be set to True if groups are unequal. Default is False.
# #     spaced : Bool, optional
# #         Spaces out data points so that they are not at an identical x value, e.g. grape bunch style. Default is False.
# #     yspace : Int or Float, optional
# #         Used in conjunction with spaced to determine spacing of data points. Default is 20.
# #     xspace : Float, optional
# #         Used in conjunction with spaced to determine spacing of data points. Default is 0.1.
# #     barfacecolor_option : Str, optional
# #         Chooses bar color option. Default is 'same' and other options are 'between' or 'individual'.
# #     barfacecolor : List of str, optional
# #         If 'between' or 'individual' is chosen for above option then number of colors needs to match number of bars or groups. Default is ['white'].
# #     baredgecolor_option : Str, optional
# #         Chooses bar edge option. Default is 'same', see notes on barfacecolor_option.
# #     baredgecolor : List of str, optional
# #         Bar edge colors. Default is ['black'], see notes on barfacecolor.
# #     baralpha : Float, optional
# #         Sets opacity of bars so must be between 0 and 1. Default is 1.
# #     scatterfacecolor_option : Str, optional
# #         Chooses scatter face option. Default is 'same', see notes on barfacecolor_option.
# #     scatterfacecolor : List of str, optional
# #         Scatter face colors. Default = ['white'], see notes on barfacecolor.
# #     scatteredgecolor_option : Str, optional
# #         Chooses scatter edge option. Default is 'same', see notes on barfacecolor_option.
# #     scatteredgecolor : List of str, optional
# #         Scatter face colors. Default is ['grey'], see notes on barfacecolor.
# #     scatterlinecolor : Str, optional
# #         Color of lines connecting related data points, should not be in a list. Default is 'grey'.
# #     scattersize : Int or Float, optional
# #         Size of datapoints. Default is 80.
# #     scatteralpha : Float, optional
# #         Sets opacity/transparency of scatter points. Default is 1.
# #     spreadscatters : Bool, optional
# #         Not currently functional. Needs to be checked.
# #     linewidth : Float, optional
# #         Width of lines. Default is 0.75.
# #     xlim : List or 2-tuple of floats, optional
# #         Sets limits of x-axis. Default is [].
# #     ylim : List or 2-tuple of floats, optional
# #         Sets limits of x-axis. Default is [].
# #     ylabel : Str, optional
# #         Sets y-axis label. Default is 'none'.
# #     xlabel : Str, optional
# #         Sets x-axis label. Default is 'none'.
# #     grouplabel : List of str, optional
# #         Sets labels for each group. Default is 'auto'.
# #     itemlabel : List of str, optional
# #         Not currently functional. Needs to be checked. Default is 'none'.
# #     barlabels : List of str, optional
# #         Sets labels for each bar. Default is [].
# #     barlabeloffset : Float, optional
# #         Sets barlabel offset relative to x baseline. Default is 0.025.
# #     grouplabeloffset : Float, optional
# #         Sets grouplabel offset relative to x baseline. Default is 0.0250.
# #     yaxisparams : None, optional
# #         Not currently functional. Needs to be checked.
# #     show_legend : Bool or str, optional
# #         Shows legend. Default is 'none'.
# #     legendloc : Str, optional
# #         Sets legend location. Default is 'upper right'.
# #     xfontsize : Int, optional
# #         Sets x-axis font size. Default is 8.
# #     ax : Matplotlib axis object, optional
# #         Axis object to plot in. If not provided, plots in new figure/axis. Default is [].
    
# #     Returns
# #     -------------
# #     ax : Matplotlib axis object
# #         Axis object of plot.
# #     barx : List of floats
# #         x-values where each bar is plotted.
# #     barlist : List of bar container objects
# #         Allows modifcation, e.g. changing of colors of individual bars. See notes.
# #     sclist : List of scatter container objects
# #         Allows modifcation as described above. See notes.
# #     """


    

    


# #     # X ticks
# #     ax.tick_params(
# #         axis='x',          # changes apply to the x-axis
# #         which='both',      # both major and minor ticks are affected
# #         bottom=False,      # ticks along the bottom edge are off
# #         top=False,
# #         labelbottom=False) # labels along the bottom edge are off
    
# #     ax.set_xticks([])






    
    
    
#     return ax, barx, barlist, sclist
      
# if __name__ == "__main__":
#     barscatter([[1, 2, 3, 4], [5, 6, 7, 8]], scatteralpha=0.2, paired=True)


# -*- coding: utf-8 -*-
"""
Created on Fri Apr 17 13:40:05 2020

@author: James Edgar McCutcheon
"""

import numpy as np
import scipy.optimize as opt
import scipy.stats as stats


def lickCalc(licks, offset = [], burstThreshold = 0.5, runThreshold = 10,
             ignorelongilis=True, longlickThreshold=0.3, minburstlength=1,
             binsize=60, histDensity = False):
    """
    Calcuates various parameters for a train of licking data including bursting 
    parameters and returns as a dictionary.

    Parameters
    ----------
    licks : List or 1D array
        Timestamps of licks (onsets). Expects in seconds. If given in ms then other
        parameters should be changed, e.g. `burstThreshold`.
    offset : List or 1D array, optional
        Timestamps of lick offsets. The default is [].
    burstThreshold : Float, optional
        Interlick threshold (in seconds) for defining bursts. The default is 0.5.
    runThreshold : Float or Int, optional
        Number of seconds separating runs of licks. The default is 10.
    ignorelongilis : Boolean, optional
        Removes ILIs > `burstThreshold` from calcuations. Useful for calculating
        intraburst frequency. The default is True.
    longlickThreshold : Float, optional
        Threshold (in seconds) for long licks. The default is 0.3.
    minburstlength : Int, optional
        Minimum number of licks to be considered a burst. Common values are either 1 or 3. The default is 1.
    binsize : Int or Float, optional
        Size of bins for constructing histogram. The default is 60.
    histDensity : Boolean, optional
        Converts histogram into a density plot rather than absolute. The default is False.

    Returns
    -------
    lickdata, a dictionary with the following keys
    'licklength' : List of floats
        Lick lengths (empty if offset times not given)
    'longlicks' : List of floats
        Licks that are greater than `longlickThreshold`
    'licks' : List of floats
        Lick onset times
    'ilis' : List of floats
        Interlick intervals
    'freq' : Float
        Mean frequency (in Hz) of intraburst licking
    'total' : Int
        Number of licks
    'bStart' : List of floats
        Timestamps of licks that start a burst
    'bInd' : List of ints
        Indices of licks that start a burst
    'bEnd' : List of floats
        Timestamps of licks that end a burst
    'bLicks' : List of ints
        Numbers of licks in each burst
    'bTime' : Array of floats
        Duration (in seconds) of each burst
    'bMean' : Float
        Mean number of licks in all bursts
    'bMean-first3' : Float
        Mean number of licks in first three bursts
    'IBIs' : List of floats
        Interburst intervals
    'rStart' : List of floats
        Timestamps for licks that start a run. Runs designated by `runThreshold`
    'rInd' : List of ints
        Indices of licks that start a run
    'rEnd' : List of ints
        Timestamps of licks that end a run
    'rLicks' : List of ints
        Number of licks per run
    'rTime' : Array of floats
        Duration (in seconds) of each run
    'rNum' : Int
        Number of runs
    'weib_alpha' : Float
        Alpha parameter from fitted Weibull function
    'weib_beta' : Float
        Beta parameter from fitted Weibull function
    'weib_rsq' : Float
        Rsquared value from fitted Weibull function
    'burstprob' : List
        xdata and ydata of cumulative burst probability
    'hist' : List
        Histogram of licks over time
    
    Notes
    ----------
    For more information on appropriate thresholds (e.g. numbers of licks or interlick intervals) see Naneix et al (2019) for more info.
        https://doi.org/10.1016/j.neuroscience.2019.10.036
    
    For more information on the Weibull function used to model burst probability see Davis (1998) DOI: 10.1152/ajpregu.1996.270.4.R793
    """
    # makes dictionary of data relating to licks and bursts
    if type(licks) != np.ndarray or type(offset) != np.ndarray:
        try:
            licks = np.array(licks)
            offset = np.array(offset)
        except:
            print('Licks and offsets need to be arrays and unable to easily convert.')
            return
    
    lickData = {}

    if len(offset) > 0:
        licks = licks[:len(offset)]
        lickData['licklength'] = offset - licks
        if min(lickData['licklength']) < 0:
            print("Some offsets precede onsets. Not doing lick length analysis.")
            lickData['licklength'] = []
            lickData['longlicks'] = []
        else:
            lickData['longlicks'] = [x for x in lickData['licklength'] if x > longlickThreshold]
    else:
        lickData['licklength'] = []
        lickData['longlicks'] = []

    lickData['licks'] = np.concatenate([[0], licks])
    lickData['ilis'] = np.diff(lickData['licks'])
    if ignorelongilis:
        lickData['ilis'] = [x for x in lickData['ilis'] if x < burstThreshold]

    lickData['freq'] = 1/np.mean([x for x in lickData['ilis'] if x < burstThreshold])
    lickData['total'] = len(licks)
    
    # Calculates start, end, number of licks and time for each BURST 
    lickData['bStart'] = [val for i, val in enumerate(lickData['licks']) if (val - lickData['licks'][i-1] > burstThreshold)]  
    lickData['bInd'] = [i for i, val in enumerate(lickData['licks']) if (val - lickData['licks'][i-1] > burstThreshold)]
    lickData['bEnd'] = [lickData['licks'][i-1] for i in lickData['bInd'][1:]]
    lickData['bEnd'].append(lickData['licks'][-1])

    lickData['bLicks'] = np.diff(lickData['bInd'] + [len(lickData['licks'])])
    
    # calculates which bursts are above the minimum threshold
    inds = [i for i, val in enumerate(lickData['bLicks']) if val>minburstlength-1]
    
    lickData['bLicks'] = removeshortbursts(lickData['bLicks'], inds)
    lickData['bStart'] = removeshortbursts(lickData['bStart'], inds)
    lickData['bEnd'] = removeshortbursts(lickData['bEnd'], inds)
      
    lickData['bTime'] = np.subtract(lickData['bEnd'], lickData['bStart'])
    lickData['bNum'] = len(lickData['bStart'])
    if lickData['bNum'] > 0:
        lickData['bMean'] = np.nanmean(lickData['bLicks'])
        lickData['bMean-first3'] = np.nanmean(lickData['bLicks'][:3])
    else:
        lickData['bMean'] = 0
        lickData['bMean-first3'] = 0
    
    lickData['IBIs'] = []
    for bEnd in lickData['bEnd'][:-1]:
        nextlick = [lick for lick in licks if lick > bEnd][0]
        lickData['IBIs'].append(nextlick - bEnd)
        
    # Calculates start, end, number of licks and time for each RUN
    lickData['rStart'] = [val for i, val in enumerate(lickData['licks']) if (val - lickData['licks'][i-1] > runThreshold)]  
    lickData['rInd'] = [i for i, val in enumerate(lickData['licks']) if (val - lickData['licks'][i-1] > runThreshold)]
    lickData['rEnd'] = [lickData['licks'][i-1] for i in lickData['rInd'][1:]]
    lickData['rEnd'].append(lickData['licks'][-1])

    lickData['rLicks'] = np.diff(lickData['rInd'] + [len(lickData['licks'])])    
    lickData['rTime'] = np.subtract(lickData['rEnd'], lickData['rStart'])
    lickData['rNum'] = len(lickData['rStart'])

    try:
        xdata, ydata = calculate_burst_prob(lickData['bLicks'])
        try:
            lickData['weib_alpha'], lickData['weib_beta'], lickData['weib_rsq'] = fit_weibull(xdata, ydata)
        except RuntimeError:
            print('Optimal fit parameters not found')
            lickData['weib_alpha'], lickData['weib_beta'], lickData['weib_rsq'] = [0, 0, 0]
        except TypeError:
            print('Optimal fit parameters not found')
            lickData['weib_alpha'], lickData['weib_beta'], lickData['weib_rsq'] = [0, 0, 0]
            
        lickData['burstprob']=[xdata, ydata]
    except ValueError:
        print('Could not calculate burst probability')
        lickData['weib_alpha'], lickData['weib_beta'], lickData['weib_rsq'] = [0, 0, 0]
        lickData['burstprob']=[[0], [0]]
    try:
        lickData['hist'] = np.histogram(lickData['licks'][1:], 
                                    range=(0, 3600), bins=int((3600/binsize)),
                                          density=histDensity)[0]
    except TypeError:
        print('Problem making histograms of lick data')
        
    return lickData

def removeshortbursts(data, inds):
    """
    Removes bursts that do not meet minimum burst criteria.

    Parameters
    ----------
    data : List
        Data with parameters of individual bursts.
    inds : List
        Indices of bursts to be removed.

    Returns
    -------
    data : List
        Burst parameters with those that did not meet criterion removed.

    """
    data = [data[i] for i in inds]
    return data

def calculate_burst_prob(bursts):
    """
    Calculates cumulative burst probability as in seminal Davis paper
    DOI: 10.1152/ajpregu.1996.270.4.R793

    Parameters
    ----------
    bursts : List
        List with lengths of bursts.

    Returns
    -------
    x : List
        x values (bin sizes).
    y : List
        y values of cumulative burst probability.

    """
    bins = np.arange(min(bursts), max(bursts))
    hist=np.histogram(bursts, bins=bins, density=True)
    cumsum=np.cumsum(hist[0])

    x = hist[1][1:]
    y = [1-val for val in cumsum]
    
    return x, y

def weib_davis(x, alpha, beta):
    '''Weibull function as used in Davis (1998) DOI: 10.1152/ajpregu.1996.270.4.R793'''
    return (np.exp(-(alpha*x)**beta))

def fit_weibull(xdata, ydata):
    '''Fits Weibull function to xdata and ydata and returns fit parameters.'''
    x0=np.array([0.1, 1])
    fit=opt.curve_fit(weib_davis, xdata, ydata, x0)
    alpha=fit[0][0]
    beta=fit[0][1]
    slope, intercept, r_value, p_value, std_err = stats.linregress(ydata, weib_davis(xdata, alpha, beta))
    r_squared=r_value**2
    
    return alpha, beta, r_squared

if __name__ == '__main__':
    print('Testing functions')
    import trompy as tp
    filename = "C:\Github\Lick-Calc-GUI\data\!2017-07-28_09h46m.Subject pcf1.06"

    data = tp.medfilereader(filename)[4][1:]

    lickdata = lickCalc(data)



# %%
# -*- coding: utf-8 -*-
"""
Created on Fri Apr 17 16:18:05 2020

@author: James Edgar McCutcheon
"""
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as mc
import matplotlib.cbook as cbook
import matplotlib.mlab as mlab
import colorsys

def setsameaxislimits(axes, axis='y'):

    """ Sets same y axis limits for all axis objects passed.
    
    Parameters
    -----------
    axes : List of Matplotlib axis objects
        One or more axis objects to be equalized.
    axis : Str, optional
        Non-functional but will be added in future. Default is "y".
    """
    axmin = []
    axmax = []
    for ax in axes:
        axmin.append(ax.get_ylim()[0])
        axmax.append(ax.get_ylim()[1])
    min_axmin = np.min(axmin)
    max_axmax = np.max(axmax)
    
    for ax in axes:
        ax.set_ylim([min_axmin, max_axmax])
        
def invisible_axes(ax):
    """ Sets axes to invisible. """
    ax.xaxis.set_visible(False)
    ax.yaxis.set_visible(False)
    for sp in ['left', 'right', 'top', 'bottom']:
        ax.spines[sp].set_visible(False)

def shadedError(ax, yarray, linecolor='black', errorcolor = 'xkcd:silver', linewidth=1,
                    linestyle="-", alpha=0.4, **kwargs):
    """ Simple shaded error plot.
    
    Parameters
    ------------
    ax : Matplotlib axis object
        Axis where data will be plotted.
    yarray : 2D array or List of list of floats
        Data to be plotted where rows are individual trials and columns are time bins.
    linecolor : Str, optional
        Color that average (mean) will be plotted in. Default is "black".
    errorcolor : Str, optional
        Color that shaded error area will be. Default is "xkcd:silver".
    linewidth : Float or int, optional
        Line width for average (mean). Default is 1.
    linestyle : Str, optional
        Line style for average. Default is solid ("-").
    alpha : Float, optional
        Transparency of error. Should be between 0 and 1. Default is 0.4.
        
    Returns
    ----------
    ax : Matplotlib axis object
    """
    yarray = np.array(yarray)
    y = np.mean(yarray, axis=0)
    yerror = np.std(yarray, axis=0)/np.sqrt(len(yarray))
    x = np.arange(0, len(y))
    ax.plot(x, y, color=linecolor, linewidth=linewidth, linestyle=linestyle)
    ax.fill_between(x, y-yerror, y+yerror, color=errorcolor, alpha=alpha)
    
    return ax

def ax2prop(axlims, n):
    """ Scales values relative to axis limits, e.g. for determining where events should be plotted on an axis.
    
    Parameters
    ------------
    axlims : Tuple or lust with 2 elements
        Normally the result of ax.get_xlims()
    n : List of floats
        Values that should be scaled relative to axis limits.
        
    Returns
    ---------
    output : List of floats
        Scaled values.
    """
    axrange = axlims[1] - axlims[0]
    output = []
    for n in n:
        output.append((n - axlims[0])/axrange)
        
    return output

def lighten_color(color, amount=0.5):
    """
    Lightens the given color by multiplying (1-luminosity) by the given amount.
    Input can be matplotlib color string, hex string, or RGB tuple.
    
    Parameters
    -------------
    color : Str
        Color to be lightened
    amount : Float, optional
        Amount to lighten color by. Default is 0.5.
        
    Examples
    -------------
    >> lighten_color('g', 0.3)
    >> lighten_color('#F034A3', 0.6)
    >> lighten_color((.3,.55,.1), 0.5)
    """

    try:
        c = mc.cnames[color]
    except:
        c = color
    c = colorsys.rgb_to_hls(*mc.to_rgb(c))
    return colorsys.hls_to_rgb(c[0], 1 - amount * (1 - c[1]), c[2])

def get_violinstats(dataset, points=100, bw_method=None):

    """ Gets stats for a violin plot.
    
    Parameters
    ------------
    dataset : List or array of floats
        Dataset for constructing violin plot.
    points : Int, optional
        Number of points for caclulating violin plot. Default is 100.
    bw_method : Str, optional
        Bandwidth method for calculating violin plot. Default is None.
        
    Returns
    ---------
    vpstats : Structured array
    
    """

    def _kde_method(X, coords):
        # fallback gracefully if the vector contains only one value
        if np.all(X[0] == X):
            return (X[0] == coords).astype(float)
        kde = mlab.GaussianKDE(X, bw_method)
        return kde.evaluate(coords)

    vpstats = cbook.violin_stats(dataset, _kde_method, points=points)
    return vpstats

if __name__ == "__main__":
    data = np.random.random([50,100])
    f, ax = plt.subplots()
    shadedError(ax, data, linestyle="--")
# %%
